/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "globals.h"
#include "wndlist.h"
#include "rule.h"

////////////////////////////////////////////////////////////////////////////////////
BOOL CRuleExp::bad() const
{
	((CRuleExp*)this)->Parse();
	return (dwFlags & fParseError) != 0;
}

BOOL CRuleExp::Compute(const const CWndData& data) const
{
	return good() && GetCount()>0 && GetHead()->Compute(data);
}

CRuleExp& CRuleExp::operator=(LPCTSTR psz)
{
	sExpression = psz;
	sParseError.Empty();
	dwFlags &= ~(fParsed|fParseError);
	return *this;
}

////////////////////////////////////////////////////////////////////////////////////
const CRule* 
CRuleSet::_FindRule(const CWndData& wndData, DWORD dwFlags, BOOL bOnlyActive/*=FALSE*/)
{
	int sz = GetSize();
	for(int i=0; i<sz; i++)
	{
		const CRule *rule = &(*this)[i];
		if( rule->stmt.rsDisable )
			continue; //skip disabled rules
		if( (rule->stmt.dwFlags & dwFlags) != dwFlags )
			continue;
		if( bOnlyActive ) {
			CRule::RuleStatement rs(dwFlags);
			if( rs.rsMinimizeWindow && !rule->minimize.bActive ) continue;
			if( rs.rsHideWindow     && !rule->hide.bActive )     continue;
			if( rs.rsCloseWindow    && !rule->close.bActive )    continue;
		}
		if( rule->expr.Compute(wndData) )
			return rule;
	}
	return NULL;
}

const CRule*
CRuleSet::_IsIgnore(const CWndData& wndData)
{
	const CRule *rule;
	return
		(rule = _FindRule(wndData, CRule::rsfIgnoreWindow(2)))?rule:
		(rule = _FindRule(wndData, CRule::rsfIgnoreWindow(1)))?rule:
		NULL;
}

BOOL
CRuleSet::IsIgnore(const CWndData& wndData)
{
	const CRule *rule = _IsIgnore(wndData);
	return rule && rule->stmt.rsIgnoreWindow == 1;
}

const CRule*
CRuleSet::_IsMinimize(const CWndData& wndData, BOOL bOnlyActive/*=FALSE*/)
{
	const CRule *rule;
	return
		(rule = _FindRule(wndData, CRule::rsfMinimizeWindow(2)))?rule:
		(rule = _FindRule(wndData, CRule::rsfMinimizeWindow(1),bOnlyActive))?rule:
		NULL;
}

int
CRuleSet::IsMinimize(const CWndData& wndData, BOOL bOnlyActive/*=FALSE*/)
{
	const CRule *rule = _IsMinimize(wndData, bOnlyActive);
	if( rule && rule->stmt.rsMinimizeWindow == 1 )
		return rule->minimize.wTimeout?SEC2TIMESLICE(rule->minimize.wTimeout):1;
	return 0;
}

const CRule*
CRuleSet::_IsHide(const CWndData& wndData, BOOL bOnlyActive/*=FALSE*/)
{
	const CRule *rule;
	return
		(rule = _FindRule(wndData, CRule::rsfHideWindow(2)))?rule:
		(rule = _FindRule(wndData, CRule::rsfHideWindow(1),bOnlyActive))?rule:
		NULL;
}


int
CRuleSet::IsHide(const CWndData& wndData, BOOL bOnlyActive/*=FALSE*/)
{
	const CRule *rule = _IsHide(wndData, bOnlyActive);
	if( rule && rule->stmt.rsHideWindow== 1 )
		return rule->hide.wTimeout?SEC2TIMESLICE(rule->hide.wTimeout):1;
	return 0;
}

const CRule*
CRuleSet::_IsClose(const CWndData& wndData, BOOL bOnlyActive/*=FALSE*/)
{
	const CRule *rule;
	return
		(rule = _FindRule(wndData, CRule::rsfCloseWindow(2)))?rule:
		(rule = _FindRule(wndData, CRule::rsfCloseWindow(1),bOnlyActive))?rule:
		NULL;
}


int
CRuleSet::IsClose(const CWndData& wndData, BOOL bOnlyActive/*=FALSE*/)
{
	const CRule *rule = _IsClose(wndData, bOnlyActive);
	if( rule && rule->stmt.rsCloseWindow== 1 )
		return rule->close.wTimeout?SEC2TIMESLICE(rule->close.wTimeout):1;
	return 0;
}


const CRule*
CRuleSet::_IsTray(const CWndData& wndData)
{
	const CRule *rule;
	return
		(rule = _FindRule(wndData, CRule::rsfTrayWindow(2)))?rule:
		(rule = _FindRule(wndData, CRule::rsfTrayWindow(1)))?rule:
		NULL;
}

BOOL
CRuleSet::IsTray(const CWndData& wndData)
{
	const CRule *rule = _IsTray(wndData);
	return rule && rule->stmt.rsTrayWindow == 1;
}

const CRule*
CRuleSet::_IsPassword(const CWndData& wndData)
{
	const CRule *rule;
	return
		(rule = _FindRule(wndData, CRule::rsfPasswordWindow(2)))?rule:
		(rule = _FindRule(wndData, CRule::rsfPasswordWindow(1)))?rule:
		NULL;
}

BOOL
CRuleSet::IsPassword(const CWndData& wndData)
{
	const CRule *rule = _IsPassword(wndData);
	return rule && rule->stmt.rsPasswordWindow == 1;
}

void CRule::RegSave(LPCTSTR pszSection)
{
	CWinApp *pApp = AfxGetApp(); ASSERT(pApp);
	HKEY hKey = pApp->GetSectionKey(_T(""));
	if( hKey ) {
		pApp->DelRegTree(hKey, pszSection);
		RegCloseKey(hKey);
	}
	pApp->WriteProfileString(pszSection, "Name",			name);
	pApp->WriteProfileString(pszSection, "Expression",		expr);
	pApp->WriteProfileInt   (pszSection, "Flags",			stmt.dwFlags);
	pApp->WriteProfileInt   (pszSection, "minimize-time",	minimize.wTimeout);
	pApp->WriteProfileInt   (pszSection, "minimize-flags",	minimize.bActive);
	pApp->WriteProfileInt   (pszSection, "hide-time",		hide.wTimeout);
	pApp->WriteProfileInt   (pszSection, "hide-flags",		hide.bActive);
	pApp->WriteProfileInt   (pszSection, "close-time",		close.wTimeout);
	pApp->WriteProfileInt   (pszSection, "close-flags",		close.bActive);
}

BOOL CRule::RegRead(LPCTSTR pszSection)
{
	CWinApp *pApp = AfxGetApp(); ASSERT(pApp);

	name =				pApp->GetProfileString(pszSection, "Name");
	expr =				pApp->GetProfileString(pszSection, "Expression");
	stmt.dwFlags =		pApp->GetProfileInt   (pszSection, "Flags",0);
	minimize.wTimeout =	pApp->GetProfileInt   (pszSection, "minimize-time",0);
	minimize.bActive =	pApp->GetProfileInt   (pszSection, "minimize-flags",0);
	hide.wTimeout =		pApp->GetProfileInt   (pszSection, "hide-time",0);
	hide.bActive =		pApp->GetProfileInt   (pszSection, "hide-flags",0);
	close.wTimeout =	pApp->GetProfileInt   (pszSection, "close-time",0);
	close.bActive =		pApp->GetProfileInt   (pszSection, "close-flags",0);
	return expr.good();
}

void CRuleSet::RegSave(LPCTSTR pszSection)
{
	CWinApp *pApp = AfxGetApp(); ASSERT(pApp);
	HKEY hKey = pApp->GetSectionKey(_T(""));
	if( hKey ) {
		pApp->DelRegTree(hKey, pszSection);
		RegCloseKey(hKey);
	}
	CString sSection;
	for(int i=0;i<GetSize();i++) {
		sSection.Format("%s\\Rule-%d", pszSection, i+1);
		(*this)[i].RegSave(sSection);
	}
	pApp->WriteProfileInt(pszSection, "Count", GetSize());
}

BOOL CRuleSet::RegRead(LPCTSTR pszSection)
{
	CWinApp *pApp = AfxGetApp(); ASSERT(pApp);
	int nCount = pApp->GetProfileInt(pszSection, "Count", -1);
	if( nCount < 0 ) return FALSE;

	CString sSection;
	for( int i=0;i<nCount;i++) {
		 SetSize(GetSize()+1);
		 sSection.Format("%s\\Rule-%d", pszSection, i+1);
		 if( !(*this)[GetSize()-1].RegRead(sSection) )
			 RemoveAt(GetSize()-1);
	}
	return TRUE;
}
