/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __RULE_H_INCLUDED__
#define __RULE_H_INCLUDED__

class CWndData;
namespace RULEEXP {
	struct OpBase {
		virtual ~OpBase() {}
		virtual BOOL Compute(const CWndData& data) const=0;
	};
	
	static int yyparse(void*);
}

inline void AFXAPI
DestructElements(RULEEXP::OpBase** pElements, int nCount )
	{ for( ;nCount>0;nCount-- ) delete *pElements++; }

class CRuleExp:
	public CList<RULEEXP::OpBase*, RULEEXP::OpBase*>
{
	friend static int RULEEXP::yyparse(void*);
	enum { fParsed=1, fParseError=2 };

	DWORD	dwFlags;
	CString sExpression;
	CString sParseError;
	
	LPCTSTR pszExpression;
	CString sTempString;
	int Lexer(void *lvalp);
	void Parse();
public:
	CRuleExp(LPCTSTR pszExpr=NULL):
		sExpression(pszExpr), dwFlags(0)
		{}
	CRuleExp(const CRuleExp& x):
		sExpression(x.sExpression), dwFlags(0)
		{}
public:
	BOOL Compute(const CWndData& data) const;
	CRuleExp& operator =(LPCTSTR psz);
	CRuleExp& operator =(const CRuleExp& x)
		{ return this->operator=(x.sExpression); }
	operator LPCTSTR() const
		{ return expression(); }

	BOOL bad() const;
	BOOL good() const
		{ return !bad(); }
	LPCTSTR error() const
		{ return sParseError; }
	LPCTSTR expression() const
		{ return sExpression; }
};

#pragma pack(1)
struct CRule {
    struct RuleStatement {
    // 0 - option is off
    // 1 - option is on
    // 2 - option is reversed
    // 3 - option error
		union {
			struct {
    			BYTE rsDisable:			1;
    			BYTE rsIgnoreWindow:	2;
    			BYTE rsMinimizeWindow:	2;
    			BYTE rsHideWindow:		2;
    			BYTE rsCloseWindow:		2;
    			BYTE rsTrayWindow:		2;
				BYTE rsPasswordWindow:	2;
			};
			DWORD dwFlags;
		};
		RuleStatement(DWORD flags=0): dwFlags(flags)
			{}
    };

    struct HideData {
    	BOOL	bActive;	// perform action on active window
    	UINT	wTimeout;
		HideData(): bActive(0), wTimeout(60)
			{}
    };                   
public:
	static DWORD rsfDisable(BYTE x=1)			{RuleStatement tmp; tmp.rsDisable=x;		return tmp.dwFlags;}
	static DWORD rsfIgnoreWindow(BYTE x=3)		{RuleStatement tmp; tmp.rsIgnoreWindow=x;   return tmp.dwFlags;}
	static DWORD rsfMinimizeWindow(BYTE x=3)	{RuleStatement tmp; tmp.rsMinimizeWindow=x; return tmp.dwFlags;}
	static DWORD rsfHideWindow(BYTE x=3)		{RuleStatement tmp; tmp.rsHideWindow=x;     return tmp.dwFlags;}
	static DWORD rsfCloseWindow(BYTE x=3)		{RuleStatement tmp; tmp.rsCloseWindow=x;    return tmp.dwFlags;}
	static DWORD rsfTrayWindow(BYTE x=3)		{RuleStatement tmp; tmp.rsTrayWindow=x;     return tmp.dwFlags;}
	static DWORD rsfPasswordWindow(BYTE x=3)	{RuleStatement tmp; tmp.rsPasswordWindow=x; return tmp.dwFlags;}
public:
	CString			name;
	CRuleExp		expr;
	RuleStatement	stmt;
	HideData		minimize;
	HideData		hide;
	HideData		close;
public:
	void RegSave(LPCTSTR pszSection);
	BOOL RegRead(LPCTSTR pszSection);
};

#pragma pack()

class CRuleSet:
	public CArray<CRule, CRule&>
{
public:
	CRuleSet()
		{}
public:
	const CRule* _FindRule(const CWndData& wndData, DWORD dwFlags, BOOL bOnlyActive=FALSE);
	const CRule* _IsIgnore(const CWndData& wndData);
	const CRule* _IsTray(const CWndData& wndData);
	const CRule* _IsPassword(const CWndData& wndData);
	const CRule* _IsMinimize(const CWndData& wndData, BOOL bOnlyActive=FALSE);
	const CRule* _IsHide(const CWndData& wndData, BOOL bOnlyActive=FALSE);
	const CRule* _IsClose(const CWndData& wndData, BOOL bOnlyActive=FALSE);
public:
	BOOL IsIgnore(const CWndData& wndData);
	BOOL IsTray(const CWndData& wndData);
	BOOL IsPassword(const CWndData& wndData);
	int  IsMinimize(const CWndData& wndData, BOOL bOnlyActive=FALSE);
	int  IsHide(const CWndData& wndData, BOOL bOnlyActive=FALSE);
	int  IsClose(const CWndData& wndData, BOOL bOnlyActive=FALSE);
public:
	void RegSave(LPCTSTR pszSection);
	BOOL RegRead(LPCTSTR pszSection);
};

#endif //__RULE_H_INCLUDED__
