/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
%{
//================================================================
#include <afx.h>
#include <afxtempl.h>
#include <malloc.h>
#include "regexp.h"
#include "rule.h"
#include "wndlist.h"

namespace RULEEXP {
//#define YYERROR_VERBOSE
#undef  TRACE
#define TRACE				1 ? (void)0 : ::AfxTrace

#define YYPARSE_PARAM		__YYPARSEPARAM
#define pParser				((::CRuleExp*)__YYPARSEPARAM)
#define yyerror(msg)		pParser->sParseError +=(msg)
#define yylex(arg1)			pParser->Lexer(arg1)
static int yyparse(void*);

enum ExprType	  { etWindowName=1, etClassName, etExeName };
enum ExpModifier  { emNoCase=1, emRegexp=2, emComplete=4, emRxCompileError=0x100 };

class OpLiteral: public OpBase {
	mutable _RxNode* m_RxCompiled;
	CString		m_sExpr;
	ExprType	m_type;  
	ExpModifier	m_modifier;
public:
	OpLiteral(LPCTSTR pszExpr, ExprType type, ExpModifier modifier):
		m_sExpr(pszExpr), m_type(type), m_modifier(modifier),
		m_RxCompiled(NULL)
		{
			if( (m_modifier & emNoCase) &&
				!(m_modifier & emRegexp) )
			{
				m_sExpr.MakeUpper();
			}
		}
	~OpLiteral()
		{
			if( m_RxCompiled )
				::RxFree(m_RxCompiled);
		}
	BOOL Compute(const CWndData& data) const;
};

class OpNot: public OpBase {
	OpBase* m_op;
public:
	OpNot(OpBase* op):
		m_op(op) {}
	BOOL Compute(const CWndData& data) const
		{
			ASSERT(m_op);
			BOOL bResult = !m_op->Compute(data);
			TRACE("%0X: !%0X == %s\n", this, m_op, bResult?"TRUE":"FALSE");
			return bResult;
		}
};

class OpAnd: public OpBase {
	OpBase* m_opLeft;
	OpBase* m_opRight;
public:
	OpAnd(OpBase* opLeft, OpBase* opRight):
		m_opLeft(opLeft), m_opRight(opRight) {}
	BOOL Compute(const CWndData& data) const
		{
			ASSERT(m_opLeft && m_opRight);
			BOOL bResult = m_opLeft->Compute(data) && m_opRight->Compute(data);
			TRACE("%0X: %0X & %0X == %s\n", this, m_opLeft, m_opRight, bResult?"TRUE":"FALSE");
			return bResult;
		}
};

class OpOr: public OpBase {
	OpBase* m_opLeft;
	OpBase* m_opRight;
public:
	OpOr(OpBase* opLeft, OpBase* opRight):
		m_opLeft(opLeft), m_opRight(opRight) {}
	BOOL Compute(const CWndData& data) const
		{
			ASSERT(m_opLeft && m_opRight);
			BOOL bResult = m_opLeft->Compute(data) || m_opRight->Compute(data);
			TRACE("%0X: %0X | %0X == %s\n", this, m_opLeft, m_opRight, bResult?"TRUE":"FALSE");
			return bResult;
		}
};

//================================================================
%}
%pure_parser
%union {
	ExpModifier		chModifier;
	ExprType		chType;
	OpBase*			pOperation;
	LPCTSTR			pszSubstr;
}

%token	<pszSubstr>		TOK_SUBSTRING
%token					TOK_LEXERROR

%type	<pOperation>	input
%type	<pOperation>	expr
%type	<chType>		type
%type	<chModifier>	modifiers
%type	<chModifier>	exprtype

%left   '|' '&'
%right  '!'

/*================================================================*/
%%/*                 start of syntax diagram                      */
/*================================================================*/

input	: input '\0'
		|	{  pParser->RemoveAll(); }
		  expr
		;

expr	: expr '&' expr
			{
				OpBase* pOp = new OpAnd($1, $3);
				pParser->AddHead($$ = pOp);
			}
		| expr '|' expr
			{
				OpBase* pOp = new OpOr($1, $3);
				pParser->AddHead($$ = pOp);
			}
		| '!' expr %prec '!'
			{
				OpBase* pOp = new OpNot($2);
				pParser->AddHead($$ = pOp);
			}
		| '(' expr ')'
			{
				$$ = $2;
			}
		| type TOK_SUBSTRING modifiers
			{
				OpBase* pOp = new OpLiteral($2, $1, $3);
				pParser->AddHead($$ = pOp);
			}
        ;

type	: /* empty */	{ $$=etWindowName; /*defaulting to window name*/}
		| 'w'			{ $$=etWindowName; }
		| 'c'			{ $$=etClassName; }
		| 'e'			{ $$=etExeName; }
		;


modifiers: /* empty */	{ $$=(ExpModifier)0; }
		| 'i'			{ $$=emNoCase; }
		| exprtype		{ $$=$1; }
		| exprtype 'i'	{ $$=(ExpModifier)($1 | emNoCase); }
		| 'i' exprtype	{ $$=(ExpModifier)($2 | emNoCase); }
		;

exprtype: 'r'			{ $$=emRegexp; }
		| 'c'			{ $$=emComplete; }
		;

/*================================================================*/
%%/*                   end of syntax diagram                      */
/*================================================================*/

BOOL OpLiteral::Compute(const CWndData& data) const
{
	TRACE("%0X: matching ", this);
	
	BOOL bResult; 
	CString sString;
	switch( m_type ) {
	case etWindowName: sString = data.GetWindowName(); break;
	case etClassName:  sString = data.GetClassName(); break;
	case etExeName:    sString = data.GetExeName(); break;
	default:
		TRACE("error. unknown expression type\n");
		return FALSE;
	}
	if( m_modifier & emRegexp ) {
	//  matching regular expression
		TRACE("regexp '%s' ",(LPCTSTR)m_sExpr);
		TRACE("with '%s' ", (LPCTSTR)sString);
		if( (m_modifier & emRxCompileError) == 0 &&
			m_RxCompiled == NULL )
		{
			m_RxCompiled = ::RxCompile(m_sExpr);
			if( !m_RxCompiled ) 
				((OpLiteral*)this)->m_modifier = (ExpModifier)(m_modifier | emRxCompileError);
		}
		if( m_RxCompiled ) {
			RxMatchRes matchRes;
			bResult = RxExec(m_RxCompiled,
				sString, sString.GetLength(), sString,
				&matchRes, m_modifier&emNoCase?NULL:RX_CASE);
		} else 
			bResult = FALSE;
	} else
	if( m_modifier & emComplete ) {
	//  matching complete string
		TRACE("complete string '%s' ",(LPCTSTR)m_sExpr);
		if( m_modifier & emNoCase ) {
			TRACE("nocase ");
			sString.MakeUpper();
		}
		TRACE("with '%s' ", (LPCTSTR)sString);
		bResult = sString == m_sExpr;
	} else {
	//  matching substring
		TRACE("substring '%s' ",(LPCTSTR)m_sExpr);
		if( m_modifier & emNoCase ) {
			TRACE("nocase ");
			sString.MakeUpper();
		}
		TRACE("with '%s' ", (LPCTSTR)sString);
		bResult = sString.Find(m_sExpr) >= 0;
	}
	TRACE("== %s\n", bResult?"TRUE":"FALSE");
	return bResult;
}

///===============================================================
}// end of namespace RULEEXP
int CRuleExp::Lexer(void *lvalp)
{
	pszExpression--;
	while(++pszExpression && *pszExpression )
	{
		switch( *pszExpression ) {
	//	skip whitespace
		case ' ':
		case '\t':
		case '\r':
		case '\n':
			continue;
	//	catch substring
		case '/': {
			sTempString.Empty();
			LPCTSTR pszNext, pszStart;
			pszStart = pszNext = (LPCTSTR)(pszExpression + 1);
			while( *pszNext ) {
				if( !(pszNext = _tcschr(pszNext, '/')) )
					return TOK_LEXERROR; // no closing slash  

				LPCTSTR pszSlash = pszNext-1;
				if( *pszSlash == '\\' ) {
					while( *pszSlash == '\\' && pszSlash >= pszStart )
						pszSlash--; pszSlash++;
					if( ((pszNext-pszSlash)%2)  ) {
						sTempString += CString(pszStart, pszSlash - pszStart);
						sTempString += CString(pszSlash+1,pszNext - pszSlash);
						pszStart = ++pszNext;
						continue;
					}
				}
				sTempString += CString(pszStart, pszNext-pszStart);
				pszExpression = pszNext + 1;
				((RULEEXP::YYSTYPE*)lvalp)->pszSubstr = sTempString;
				return TOK_SUBSTRING;
			}}
		default:
			return (int)*pszExpression++;
		}
	}
	return 0;
}

void CRuleExp::Parse()
{
	if( dwFlags & fParsed )
		return;

	pszExpression = sExpression;
	if( RULEEXP::yyparse(this) != 0 )
		dwFlags |= fParseError;
	dwFlags |= fParsed;
}

