/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __SHLCACHE_H_INCLUDED__
#define __SHLCACHE_H_INCLUDED__

class CShlItem;
class CShlFolder;
class CShlRoot;
CString GetKeyName(DWORD wHotKey);

class CShlItem:
//	public CShlItem,
	public CPidl
{
	friend class CShlFolder;
	friend class CShlRoot;
protected:
	CShlRoot*	m_pRoot;

	HICON	 m_hIcon;
	WORD	 m_wHotKey;
	CString	 m_sDisplayName;
	CString	 m_sType;
	UINT	 m_nMenuID;

	virtual void GetData();
public://construction
	CShlItem(CShlRoot* pRoot);
	CShlItem(CShlRoot* pRoot, LPITEMIDLIST pidl);
	virtual ~CShlItem();
public://virtuals
	virtual int IsFolder() const
		{ return FALSE; };
	virtual HICON GetIcon() const;
	virtual WORD GetHotKey() const;
	virtual LPCTSTR GetDisplayName() const;
	virtual LPCTSTR GetTypeName() const;
	virtual UINT GetMenuID() const
		{ return m_nMenuID; }
	virtual BOOL Invoke(LPCTSTR pszVerb=NULL) const; 
	virtual CShlRoot* GetRoot() const
		{ ASSERT(m_pRoot); return m_pRoot; }
};

inline
int AFXAPI PxCompareElements(CShlItem* const *pElement1, CShlItem* const *pElement2);
inline 
void AFXAPI DestructElements(CShlItem** pItems, int nCount)
{
	for( ;nCount>0;nCount-- ) delete *pItems++;
}

class CShlFolder:
	public CShlItem,
	public CPxArray<CShlItem*,CShlItem*>
{
	friend CShlItem;
protected:
	enum flags { stickyName     = 1,
		   stickyIcon     = 2,
		   stickyKey      = 4,
		   errNotifyError = 8 // installation of Change Notify has failed
	};

	DWORD		     m_dwFlags;
	IShellFolderPtr  m_pFolder;
	CPxChangeNotify  m_Notify;

	void GetData();
public://construction
	CShlFolder(CShlRoot* pRoot);
	CShlFolder(CShlRoot* pRoot, LPITEMIDLIST pidl);
	CShlFolder(CShlRoot* pRoot, int nSpecialFolder);
	virtual ~CShlFolder();
public://virtuals
	int IsFolder() const
		{ return TRUE; }
public:
	void SetIcon(HICON hNewIcon);
	void SetDisplayName(LPCTSTR pszNewName);
	void SetHotKey(WORD wHotKey);

	void Attach(LPITEMIDLIST pidl);
	void Attach(int nSpecialFolder);
	BOOL Refresh();
	virtual void Flush();
	const CPxChangeNotify& GetNotify() const
		{ return m_Notify; }
};

class CShlRoot:
	public CShlFolder
{
	friend int AFXAPI PxCompareElements(CShlItem* const *, CShlItem* const *);
	friend class CShlFolder;
protected:
	UINT	m_nFirstMenuID;
	UINT	m_nNextMenuID;
	UINT	m_nNestLevel;
	UINT	m_nOptions;
	CString m_sFilemask;
	CArray<CString, CString&> m_aFileMask;

	UINT GetItemLevel(CShlItem* item);
	UINT GetNextMenuID()  { return m_nNextMenuID++; }
	BOOL FileMatch(LPCITEMIDLIST pidl);
	int  CompareItems(CShlItem* item1, CShlItem* item2);
public://construction
	CShlRoot();
	CShlRoot(LPITEMIDLIST pidl);
	CShlRoot(int nSpecialFolder);
public://virtuals
	void Flush()
		{
			CShlFolder::Flush();
			m_nNextMenuID = m_nFirstMenuID;
			m_nMenuID     = GetNextMenuID();
		}
public:
	static UINT defFirstMenuID;
	enum options {
		optExpandLinks		= 1,
		optExpandRemovable	= 2,
		optShowHidden		= 4,
		optFoldersOnTop		= 8,
		optSortByType		= 16,
		optTrayIcon			= 32
	};
	enum defaults {
		defNestLevel = 3,
		defOptions = optExpandLinks|optFoldersOnTop
	};

	void SetFirstMenuID(UINT nFirstID=defFirstMenuID)
		{ m_nFirstMenuID = nFirstID; Flush(); }

	void SetOptions(UINT nOptions=defOptions)
		{ m_nOptions = nOptions; Flush(); }
	UINT GetOptions() const
		{ return m_nOptions; }

	void SetNestLevel(UINT nNestLevel=defNestLevel)
		{ m_nNestLevel = nNestLevel; Flush(); }
	UINT GetNestLevel()	const
		{ return m_nNestLevel; }

	void SetFileMask(LPCTSTR pszMask);
	CString GetFileMask() const
		{ return m_sFilemask; }

	CShlItem* GetItemByID(UINT nMenuID) const;
	void RegSave(LPCTSTR pszSection) const;
	BOOL RegRead(LPCTSTR pszSection);

	CShlRoot& operator =(const CShlRoot&);
};

template<>
int AFXAPI PxCompareElements<CShlItem*>(CShlItem* const *pElement1, CShlItem* const *pElement2)
{
	ASSERT( (*pElement1)->GetRoot() == (*pElement2)->GetRoot() );
	return ((CShlItem*)(*pElement1))->GetRoot()->CompareItems(
		(CShlItem*)(*pElement1),
		(CShlItem*)(*pElement2));
}

#endif //__SHLCACHE_H_INCLUDED__
